@@ -10,12 +10,11 @@ use lib '../XMLCompile/lib', '../LogReport/lib', '../XMLTester/lib';
 
 package TestTools;
 use vars '$VERSION';
-$VERSION = '2.35';
+$VERSION = '2.36';
 
 use base 'Exporter';
 
 use Test::More;
-use Test::Deep   qw/cmp_deeply/;
 
 use Log::Report  qw/try/;
 use Data::Dumper qw/Dumper/;
@@ -67,7 +66,7 @@ sub reader_dump($$$)
     my $h = $r->($xml);
     ok(defined $h, 'processed via dumped source');
  
-    cmp_deeply($h, $hash, "dump and direct trees");
+    is_deeply($h, $hash, "dump and direct trees");
 }
 
 
